package cn.iocoder.yudao.module.order.service.order;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseIn.OrderWarehouseInDO;
import cn.iocoder.yudao.module.order.vo.orderLocation.*;
import cn.iocoder.yudao.module.order.dal.dataobject.orderLocation.OrderLocationDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 订单仓位 Service 接口
 *
 * @author 系统管理员
 */
public interface OrderLocationService extends IService<OrderLocationDO> {


    /**
     * 创建订单仓位
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOrderLocation(@Valid OrderLocationCreateReqVO createReqVO);



    /**
     * 更新订单仓位
     * @param updateReqVO 更新信息
     */
    void updateOrderLocation(@Valid OrderLocationUpdateReqVO updateReqVO);

    /**
     * 更新入仓仓位
     * @param orderLocationUpdateReqSummaryVO 更新信息
     */
    void updateWarehouseInLocation(@Valid OrderLocationUpdateReqSummaryVO orderLocationUpdateReqSummaryVO);

    /**
     * 删除订单仓位
     * @param id 编号
     */
    void deleteOrderLocation(Long id);

    /**
     * 根据订单的入仓id删除订单储位
     *
     * @param orderId
     * @param orderItemId
     * @param warehouseInId
     */
    void deleteIfPresent(Long orderId, Long orderItemId, Long warehouseInId);

    /**
     * 获得订单仓位
     * @param id 编号
     * @return 订单仓位
     */
    OrderLocationDO getOrderLocation(Long id);

    /**
     * 获得订单仓位列表
     * @param ids 编号
     * @return 订单仓位列表
     */
    List<OrderLocationDO> getOrderLocationList(Collection<Long> ids);

    /**
     * 根据orderId获得订单仓位列表
     * @param orderId 订单id
     * @return 订单仓位列表
     */
    List<OrderLocationDO> getOrderLocationListByOrderId(Long orderId);

    /**
     * 根据warehouseInId获得订单仓位列表
     * @param warehouseInId 入仓纪录id
     * @return 订单仓位列表
     */
    List<OrderLocationDO> getOrderLocationListByWarehouseInId(Long warehouseInId);


    /**
     * 查询此订单的所有储位，然后过滤出入仓相关的储位
     *
     * @param orderId
     * @param filterOrderWarehouseInDOList
     * @return
     */
    List<OrderLocationBackVO> getOrderLocationBackVO(Long orderId, List<OrderWarehouseInDO> filterOrderWarehouseInDOList);

    List<OrderLocationBackVO> filterByWarehouseInList(List<OrderLocationDO> orderLocationDOList, List<OrderWarehouseInDO> orderWarehouseInDOList);

    List<OrderLocationBackVO> filterByWarehouseIn(List<OrderLocationDO> orderLocationListByOrderId, OrderWarehouseInDO orderWarehouseInDO);

    /**
     * 获得订单仓位分页
     * @param page 分页查询
     * @param query 查询
     * @return 订单仓位分页
     */
    PageResult<OrderLocationDO> getOrderLocationPage(OrderLocationQueryVO query, PageVO page);

    /**
     * 获得订单仓位列表, 用于 Excel 导出
     * @param query 查询
     * @return 订单仓位列表
     */
    List<OrderLocationDO> getOrderLocationList(OrderLocationQueryVO query);


    /**
     * 批量修改储位
     * @param orderLocationCreateReqVOList  修改参数
     */
    void batchOrderUpdate(List<OrderLocationCreateReqVO> orderLocationCreateReqVOList);

    void tallyUpdate(Long orderId, List<OrderLocationCreateReqVO> orderLocationCreateReqVOList);
}
